@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2003-2008 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2003-2009 Hewlett-Packard Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 __version__ = '5.0'
 __title__ = 'HPLIP Installer'
 __mod__ = 'hplip-install'
-__doc__ = "Installer for HPLIP tarball."
+__doc__ = "Installer for HPLIP tarball (called automatically after invoking the .run file)."
 
 
 # Std Lib
@@ -51,11 +51,16 @@ USAGE = [(__doc__, "", "name", True),
          utils.USAGE_HELP,
          utils.USAGE_SPACE,
          utils.USAGE_SPACE,
-         ("[OPTIONS] (FOR TESTING ONLY)", "", "header", False),
+         ("[OPTIONS] (FOR TESTING ONLY/ADVANCED)", "", "header", False),
          ("Force install of all dependencies:", "-x", "option", False),
          ("Force unknown distro mode:", "-d", "option", False),
          ("Force installation of Qt4 support:", "--qt4 (same as --enable=qt4)", "option", False),
-         ("Force configure enable/disable flag:", "--enable=<flag> or --disable=<flag>, where <flag> is 'fax-build', 'qt4', pp-build', etc. See ./configure --help for more info.", "option", False),
+         ("Force disable Qt4 support:", "--no-qt4 (same as --disable=qt4", "option", False),
+         ("Force installation of Qt3 support:", "--qt3 (same as --enable=qt3)", "option", False),
+         ("Force disable Qt3 support:", "--no-qt3 (same as --disable=qt3", "option", False),
+         ("Force installation of PolicyKit support:", "--policykit (same as --enable=policykit)", "option", False),
+         ("Force disable PolicyKit support:", "--no-policykit (same as --disable=policykit)", "option", False),
+         ("Force configure enable/disable flag:", "--enable=<flag> or --disable=<flag>, where <flag> is 'fax-build', 'qt4', 'pp-build', etc. See ./configure --help for more info.", "option", False),
         ]
 
 def usage(typ='text'):
@@ -83,8 +88,9 @@ disable = []
 try:
     opts, args = getopt.getopt(sys.argv[1:], 'hl:giatxdq:nr:b',
         ['help', 'help-rest', 'help-man', 'help-desc', 'gui', 'lang=',
-        'logging=', 'interactive', 'auto', 'text', 'qt4',
-        'network', 'retries=', 'enable=', 'disable='])
+        'logging=', 'interactive', 'auto', 'text', 'qt4', 'qt3',
+        'network', 'retries=', 'enable=', 'disable=',
+        'no-qt3', 'no-qt4', 'policykit', 'no-policykit', 'debug'])
 
 except getopt.GetoptError, e:
     log.error(e.msg)
@@ -116,7 +122,7 @@ for o, a in opts:
         if not log.set_level(log_level):
             usage()
 
-    elif o == '-g':
+    elif o in ('-g', '--debug'):
         log.set_level('debug')
 
     elif o in ('-i', '--interactive', '--text', '-t'):
@@ -153,6 +159,41 @@ for o, a in opts:
             log.error("Duplicate configuration flag: %s" % a)
             sys.exit(1)
 
+    elif o == '--no-qt4':
+        if 'qt4' not in disable and 'qt4' not in enable:
+            disable.append('qt4')
+        else:
+            log.error("Duplicate configuration flag: %s" % a)
+            sys.exit(1)
+
+    elif o == '--qt3':
+        if 'qt3' not in enable and 'qt3' not in disable:
+            enable.append('qt3')
+        else:
+            log.error("Duplicate configuration flag: %s" % a)
+            sys.exit(1)
+
+    elif o == '--no-qt3':
+        if 'qt3' not in disable and 'qt3' not in enable:
+            disable.append('qt3')
+        else:
+            log.error("Duplicate configuration flag: %s" % a)
+            sys.exit(1)
+
+    elif o == '--policykit':
+        if 'policykit' not in enable and 'policykit' not in disable:
+            enable.append('policykit')
+        else:
+            log.error("Duplicate configuration flag: %s" % a)
+            sys.exit(1)
+
+    elif o == '--no-policykit':
+        if 'policykit' not in disable and 'policykit' not in enable:
+            disable.append('policykit')
+        else:
+            log.error("Duplicate configuration flag: %s" % a)
+            sys.exit(1)
+
     elif o == '--enable':
         if a not in enable and a not in disable:
             enable.append(a)
@@ -201,25 +242,6 @@ utils.log_title(__title__, __version__, True)
 log.info("Installer log saved in: %s" % log.bold(log_file))
 log.info("")
 
-bb_build_pat = re.compile("BB_BUILD\s*=\s*(.*)", re.I)
-bb_build_value = False
-try:
-    bb_build = file('./bb_build.inc')
-except IOError:
-    pass
-else:
-    input = bb_build.read()
-    bb_build.close()
-    for x in input.splitlines():
-        match = bb_build_pat.match(x)
-        if match is not None:
-            value = match.group(1)
-            bb_build_value = utils.to_bool(value)
-            break
-
-    if bb_build_value and not restricted_override:
-        log.error("This is a restricted build. The installer is disabled. Exiting.")
-        sys.exit(1)
 
 try:
     from installer import text_install
