@@ -195,6 +195,10 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) service_to_channel(mud
    {
       *index = HPMUD_SOAPFAX_CHANNEL;
    }
+   else if (strncasecmp(sn, "hp-marvell-scan", 15) == 0)
+   {
+      *index = HPMUD_MARVELL_SCAN_CHANNEL;
+   }
    /* All the following services require MLC/1284.4. */
    else if (pd->io_mode == HPMUD_RAW_MODE || pd->io_mode == HPMUD_UNI_MODE)
    {
