@@ -215,9 +215,12 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) jd_get_device_id(mud_d
    pthread_mutex_lock(&pd->mutex);
 
    *len = device_id(pd->ip, pd->port, pd->id, sizeof(pd->id));  /* get new copy and cache it  */ 
-   memcpy(buf, pd->id, *len > size ? size : *len); 
 
-   stat = HPMUD_R_OK;
+   if (*len)
+   {
+      memcpy(buf, pd->id, *len > size ? size : *len); 
+      stat = HPMUD_R_OK;
+   }
 
    pthread_mutex_unlock(&pd->mutex);
    return stat;
@@ -444,6 +447,20 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) jd_s_channel_open(mud_
             goto bugout;  
          }
          break; 
+      case HPMUD_MARVELL_SCAN_CHANNEL:
+         port = 8290;  /* same as ScanPort1[1] */
+         pin.sin_port = htons(port);
+         if ((pc->socket = socket(AF_INET, SOCK_STREAM, 0)) == -1) 
+         {  
+            BUG("unable to open marvell-scan port %d: %m %s\n", port, pd->uri);  
+            goto bugout;  
+         }  
+         if (connect(pc->socket, (struct sockaddr *)&pin, sizeof(pin)) == -1) 
+         {  
+            BUG("unable to connect to marvell-scan port %d: %m %s\n", port, pd->uri);  
+            goto bugout;  
+         }
+         break; 
       case HPMUD_PML_CHANNEL:
          /* Do nothing here, use GetPml/SetPml instead of ReadData/WriteData. */
          break;
