@@ -74,6 +74,37 @@ public:
         return FALSE;
     }
 
+    virtual DRIVER_ERROR SetHint (PRINTER_HINT eHint, int iValue)
+    {
+        switch (eHint)
+        {
+            case PAGES_IN_DOC_HINT:
+            {
+                m_iNumPages = iValue;
+                break;
+            }
+            case SPEED_MECH_HINT:
+            {
+                return SendSpeedMechCmd (iValue);
+            }
+            case EXTRA_DRYTIME_HINT:
+            {
+                m_cExtraDryTime = (BYTE) (iValue & 0xFF);
+                break;
+            }
+            default:
+                break;
+        }
+        return NO_ERROR;
+    }
+    virtual int GetHint (PRINTER_HINT eHint)
+    {
+        if (eHint == EXTRA_DRYTIME_HINT)
+        {
+            return (int) m_cExtraDryTime;
+        }
+        return 0;
+    }
 protected:
 
 #ifdef APDK_HP_UX
@@ -89,6 +120,11 @@ protected:
 
 private:
     BOOL IsPCL3DriverwareAvailable();
+    virtual DRIVER_ERROR SendSpeedMechCmd (BOOL bLastPage)
+    {
+        return NO_ERROR;
+    }
+    BYTE    m_cExtraDryTime;
 }; //DJ9xxVIP
 
 
@@ -122,6 +158,10 @@ class DJ9902400Mode : public PrintMode
 {
 public:
     DJ9902400Mode ();
+    virtual inline BOOL MediaCompatible (MEDIATYPE eMedia)
+    {
+        return (eMedia == pmMediaType || eMedia == MEDIA_PHOTO);
+    }
 }; // DJ9902400Mode
 
 class DJ990DraftMode : public PrintMode
