@@ -384,6 +384,10 @@ class LJFastRasterProxy;
 class LJZjsMonoProxy;
 #endif
 
+#ifdef APDK_LJZJS_COLOR
+class LJZjsColorProxy;
+#endif
+
 #ifdef APDK_LJM1005
 class LJM1005Proxy;
 class LJP1XXXProxy;
@@ -540,6 +544,10 @@ public:
     static LJZjsMonoProxy s_LJZjsMonoProxy;
 #endif
 
+#ifdef APDK_LJZJS_COLOR
+    static LJZjsColorProxy s_LJZjsColorProxy;
+#endif
+
 #ifdef APDK_LJM1005
     static LJM1005Proxy s_LJM1005Proxy;
     static LJP1XXXProxy s_LJP1XXXProxy;
