@@ -1,7 +1,7 @@
 /*****************************************************************************\
   LJJetReady.h : Interface for the LJJetReady class
 
-  Copyright (c) 1996 - 2001, Hewlett-Packard Co.
+  Copyright (c) 1996 - 2008, Hewlett-Packard Co.
   All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
@@ -49,7 +49,8 @@ enum COMPRESS_MODE  {	COMPRESS_MODE0 = 0,
 						COMPRESS_MODE9 = 9,
 						COMPRESS_MODE_AUTO = 10,
 						COMPRESS_MODE_JPEG = 11,
-						COMPRESS_MODE_LJ =   12
+						COMPRESS_MODE_LJ =   12,
+                        COMPRESS_MODE_GRAFIT = 16
 					};
 
 class LJJetReady : public Printer
@@ -207,7 +208,7 @@ protected:
     //----------------------------------------------------------------
     // IJG dest manager callback
     //----------------------------------------------------------------
-    static void jpeg_flush_output_buffer_callback(BYTE* buffer, DWORD size);
+    static void jpeg_flush_output_buffer_callback(JOCTET *outbuf, BYTE* buffer, DWORD size);
 
 private:
     Printer*            thePrinter;
@@ -233,6 +234,7 @@ public:
         "hp color LaserJet 3500\0"
 		"hp color LaserJet 3550\0"
         "hp color LaserJet 3600\0"
+        "Hp Color LaserJet CP151\0"
 #ifdef APDK_MLC_PRINTER
 #endif
     ) {m_iPrinterType = eLJJetReady;}
