@@ -77,6 +77,9 @@
 #ifdef APDK_LJZJS_MONO
 #include "ljzjsmono.h"
 #endif
+#ifdef APDK_LJZJS_COLOR
+#include "ljzjscolor.h"
+#endif
 #ifdef APDK_LJM1005
 #include "ljm1005.h"
 #include "ljp1xxx.h"
@@ -112,6 +115,10 @@ LJFastRasterProxy DeviceRegistry::s_LJFastRasterProxy;
 LJZjsMonoProxy DeviceRegistry::s_LJZjsMonoProxy;
 #endif
 
+#ifdef APDK_LJZJS_COLOR
+LJZjsColorProxy DeviceRegistry::s_LJZjsColorProxy;
+#endif
+
 #ifdef APDK_LJM1005
 LJM1005Proxy DeviceRegistry::s_LJM1005Proxy;
 LJP1XXXProxy DeviceRegistry::s_LJP1XXXProxy;
