@@ -32,6 +32,8 @@
 #include "common.h"
 #include "mfpdtf.h"
 #include "hpip.h"
+#include "scl.h"
+#include "pml.h"
 
 /************************************************************************************/
 
@@ -101,9 +103,9 @@ struct  hpaioScanner_s
 {
         char *tag;   /* handle identifier */
         char deviceuri[128];
-        int deviceid;
-        int scan_channelid;
-        int cmd_channelid;
+        HPMUD_DEVICE deviceid;
+        HPMUD_CHANNEL scan_channelid;
+        HPMUD_CHANNEL cmd_channelid;
         
         struct hpaioScanner_s * prev;
         struct hpaioScanner_s * next;
@@ -187,6 +189,7 @@ struct  hpaioScanner_s
         int ip_done;
         int page_done;
         int upload_state;                                 /* last pml upload state */
+        int user_cancel;                                  /* user cancelled operation */
 
         struct 
         {
@@ -242,6 +245,7 @@ struct  hpaioScanner_s
 };
 
 typedef struct hpaioScanner_s * hpaioScanner_t;
+typedef struct hpaioScanner_s HPAIO_RECORD;
 
 #define UNDEFINED_MODEL(hpaio) (!hpaio->saneDevice.model)
 
@@ -267,8 +271,8 @@ typedef struct hpaioScanner_s * hpaioScanner_t;
     } \
   } while(0)
 
-SANE_Status hpaioScannerToSaneStatus( hpaioScanner_t hpaio );
-SANE_Status hpaioScannerToSaneError( hpaioScanner_t hpaio );
-void sane_hpaio_cancel( SANE_Handle handle );
+SANE_Status __attribute__ ((visibility ("hidden"))) hpaioScannerToSaneStatus( hpaioScanner_t hpaio );
+SANE_Status __attribute__ ((visibility ("hidden"))) hpaioScannerToSaneError( hpaioScanner_t hpaio );
+void sane_hpaio_cancel(SANE_Handle handle);
 
 #endif
