@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2003-2008 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2003-2009 Hewlett-Packard Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 __version__ = '9.0'
 __title__ = 'PC Sendfax Utility'
 __mod__ = 'hp-sendfax'
-__doc__ = "Allows for sending faxes from the PC using HPLIP supported multifunction printers." 
+__doc__ = "PC send fax for HPLIP supported multifunction printers."
 
 # Std Lib
 import sys
@@ -48,20 +48,20 @@ group_list = []
 prettyprint = False
 
 mod = module.Module(__mod__, __title__, __version__, __doc__, None,
-                    (GUI_MODE, NON_INTERACTIVE_MODE), 
+                    (GUI_MODE, NON_INTERACTIVE_MODE),
                     (UI_TOOLKIT_QT3, UI_TOOLKIT_QT4))
-                    
+
 mod.setUsage(module.USAGE_FLAG_DEVICE_ARGS | module.USAGE_FLAG_SUPRESS_G_DEBUG_FLAG,
     extra_options=[
     ("Specify the fax number(s):", "-f<number(s)> or --faxnum=<number(s)> or --fax-num=<number(s)>  or --num=<number(s)>(-n only)", "option", False),
-    ("Specify the recipient(s):", "-r<recipient(s)> or --recipient=<recipient(s)> (-n only)", "option", False), 
+    ("Specify the recipient(s):", "-r<recipient(s)> or --recipient=<recipient(s)> (-n only)", "option", False),
     ("Specify the groups(s):", "--group=<group(s)> or --groups=<group(s)> (-n only)", "option", False) ],
     see_also_list=['hp-faxsetup', 'hp-fab'])
 
 opts, device_uri, printer_name, mode, ui_toolkit, loc = \
     mod.parseStdOpts('f:r:g:',
                      ['faxnum=', 'fax-num=', 'recipient=', 'group=',
-                      'groups=', 'gg'], 
+                      'groups=', 'gg'],
                       supress_g_debug_flag=True)
 
 for o, a in opts:
@@ -88,14 +88,10 @@ for o, a in opts:
 if not prop.fax_build:
     log.error("Fax is disabled (turned off during build). Exiting")
     sys.exit(1)
-    
-printer_name, device_uri = mod.getPrinterName(printer_name, device_uri, 
+
+printer_name, device_uri = mod.getPrinterName(printer_name, device_uri,
     filter={'fax-type': (operator.gt, 0)}, back_end_filter=['hpfax'])
 
-#if printer_name is not None:
-#    mod.lockInstance(printer_name)
-
-
 if mode == GUI_MODE:
     if ui_toolkit == 'qt3':
         if not utils.canEnterGUIMode():
@@ -110,16 +106,19 @@ if mode == GUI_MODE:
     if ui_toolkit == 'qt3':
         app = None
         sendfax = None
-        from qt import *
 
-        # UI Forms
-        from ui.faxsendjobform import FaxSendJobForm
+        try:
+            from qt import *
+            from ui.faxsendjobform import FaxSendJobForm
+        except ImportError:
+            log.error("Unable to load Qt3 support. Is it installed?")
+            sys.exit(1)
 
         # create the main application object
         app = QApplication(sys.argv)
 
         if loc is None:
-            loc = user_cfg.ui.get("loc", "system")
+            loc = user_conf.get('ui', 'loc', 'system')
             if loc.lower() == 'system':
                 loc = str(QTextCodec.locale())
                 log.debug("Using system locale: %s" % loc)
@@ -161,7 +160,7 @@ if mode == GUI_MODE:
         if os.geteuid() == 0:
             log.error("You must not be root to run this utility.")
 
-            QMessageBox.critical(None, 
+            QMessageBox.critical(None,
                                  "HP Device Manager - Send Fax",
                                  "You must not be root to run hp-sendfax.",
                                   QMessageBox.Ok,
@@ -171,9 +170,9 @@ if mode == GUI_MODE:
             sys.exit(1)
 
         # TODO: Fix instance lock
-        sendfax = FaxSendJobForm(device_uri,  
-                                 printer_name, 
-                                 mod.args) 
+        sendfax = FaxSendJobForm(device_uri,
+                                 printer_name,
+                                 mod.args)
 
         app.setMainWidget(sendfax)
 
@@ -189,17 +188,20 @@ if mode == GUI_MODE:
             pass
 
     else: # qt4
-        try:
+        #try:
+        if 1:
             from PyQt4.QtGui import QApplication
             from ui4.sendfaxdialog import SendFaxDialog
-        except ImportError:
+        #except ImportError:
+        if 0:
             log.error("Unable to load Qt4 support. Is it installed?")
-            sys.exit(1)            
+            sys.exit(1)
 
         app = QApplication(sys.argv)
 
-        toolbox = SendFaxDialog(None, device_uri, mod.args) # TODO: Add instance lock in GUI code
-        toolbox.show()
+        dlg = SendFaxDialog(None, printer_name, device_uri, mod.args)
+        dlg.show()
+
         try:
             log.debug("Starting GUI loop...")
             app.exec_()
@@ -224,7 +226,7 @@ else: # NON_INTERACTIVE_MODE
         except ImportError:
             # This can fail on Python < 2.3 due to the datetime module
             log.error("Fax address book disabled - Python 2.3+ required.")
-            sys.exit(1)    
+            sys.exit(1)
 
         db =  fax.FaxAddressBook() # FAB instance
 
@@ -235,7 +237,7 @@ else: # NON_INTERACTIVE_MODE
             sys.exit(1)
 
         dbus_avail, service, session_bus = device.init_dbus()
-        
+
         if not dbus_avail or service is None:
             log.error("Unable to initialize dBus. PC send fax requires dBus and hp-systray support. Exiting.")
             sys.exit(1)
@@ -283,8 +285,9 @@ else: # NON_INTERACTIVE_MODE
 
         for p in recipient_list:
             a = db.get(p)
-            phone_num_list.append(a)
-            log.debug("Name=%s Number=%s" % (a['name'], a['fax']))
+            if a['fax']:
+                phone_num_list.append(a)
+                log.debug("Name=%s Number=%s" % (a['name'], a['fax']))
 
         for p in faxnum_list:
             phone_num_list.append({'fax': p, 'name': u'Unknown'})
@@ -296,8 +299,6 @@ else: # NON_INTERACTIVE_MODE
             mod.usage(error_msg=["No recipients specified. Please use -f, -r, and/or -g to specify recipients."])
 
         allowable_mime_types = cups.getAllowableMIMETypes()
-        allowable_mime_types.append("application/hplip-fax")
-        allowable_mime_types.append("application/x-python")
 
         for f in mod.args:
             path = os.path.realpath(f)
@@ -319,7 +320,7 @@ else: # NON_INTERACTIVE_MODE
         #ok, lock_file = utils.lock_app('%s-%s' % (__mod__, printer_name), True)
         mod.lockInstance(printer_name)
 
-        try:            
+        try:
             ppd_file = cups.getPPD(printer_name)
 
             if ppd_file is not None and os.path.exists(ppd_file):
@@ -357,7 +358,7 @@ else: # NON_INTERACTIVE_MODE
                     file_list.append((f, mime_type, "", "", pages))
 
                 else:
-                    all_pages = True 
+                    all_pages = True
                     page_range = ''
                     page_set = 0
                     nup = 1
@@ -375,22 +376,27 @@ else: # NON_INTERACTIVE_MODE
                     if nup > 1:
                         cups.addOption('number-up=%d' % nup)
 
-                    cups_printers = cups.getPrinters()
-                    printer_state = cups.IPP_PRINTER_STATE_STOPPED
-                    for p in cups_printers:
-                        if p.name == printer_name:
-                            printer_state = p.state
+                    while True:
 
-                    log.debug("Printer state = %d" % printer_state)
+                        cups_printers = cups.getPrinters()
+                        printer_state = cups.IPP_PRINTER_STATE_STOPPED
+                        for p in cups_printers:
+                            if p.name == printer_name:
+                                printer_state = p.state
 
-                    if printer_state == cups.IPP_PRINTER_STATE_IDLE:
-                        log.debug("Printer name = %s file = %s" % (printer_name, path))
-                        sent_job_id = cups.printFile(printer_name, path, os.path.basename(path))
-                        log.info("\nRendering file '%s' (job %d)..." % (path, sent_job_id))
-                        log.debug("Job ID=%d" % sent_job_id)
-                    else:
-                        log.error("The CUPS queue for '%s' is in a stopped or busy state. Please check the queue and try again." % printer_name)
-                        sys.exit(1)
+                        log.debug("Printer state = %d" % printer_state)
+
+                        if printer_state == cups.IPP_PRINTER_STATE_IDLE:
+                            log.debug("Printer name = %s file = %s" % (printer_name, path))
+                            sent_job_id = cups.printFile(printer_name, path, os.path.basename(path))
+                            log.info("\nRendering file '%s' (job %d)..." % (path, sent_job_id))
+                            log.debug("Job ID=%d" % sent_job_id)
+                            break
+                        elif printer_state == cups.IPP_PRINTER_STATE_PROCESSING:
+                            log.debug("Waiting for CUPS queue '%s' to become idle." % printer_name)
+                        else:
+                            log.error("The CUPS queue for '%s' is in a stopped or busy state (%d). Please check the queue and try again." % (printer_name, printer_state))
+                            sys.exit(1)
 
                     cups.resetOptions()
 
@@ -398,7 +404,7 @@ else: # NON_INTERACTIVE_MODE
                     # Wait for fax to finish rendering
                     #
 
-                    end_time = time.time() + 120.0 
+                    end_time = time.time() + 120.0
                     while time.time() < end_time:
                         log.debug("Waiting for fax...")
                         try:
@@ -470,7 +476,7 @@ else: # NON_INTERACTIVE_MODE
                     log.error("Device is busy or in an error state (code=%d). Please wait for the device to become idle or clear the error and try again." % dev.error_state)
                     sys.exit(1)
 
-                user_cfg.last_used.device_uri = dev.device_uri
+                user_conf.set('last_used', 'device_uri', dev.device_uri)
 
                 log.debug("File list:")
 
@@ -484,7 +490,7 @@ else: # NON_INTERACTIVE_MODE
 
                 log.info("\nSending fax...")
 
-                if not dev.sendFaxes(phone_num_list, file_list, "", 
+                if not dev.sendFaxes(phone_num_list, file_list, "",
                                      "", None, False, printer_name,
                                      update_queue, event_queue):
 
@@ -553,9 +559,9 @@ else: # NON_INTERACTIVE_MODE
 
         finally:
             mod.unlockInstance()
-        
+
     except KeyboardInterrupt:
-        log.error("User exit")    
-    
+        log.error("User exit")
+
 log.info("")
 log.info("Done.")
