@@ -38,6 +38,16 @@ from prnt import cups
 
 
 if __name__ == '__main__':
+
+    # Create a new session ID for the tray.  This disassociates the
+    # tray from the controlling terminal so that it cannot receive
+    # keyboard interrupts.
+    #
+    # Only do this if we aren't already a session leader.  This test
+    # only succeeds if we are executed from hp-toolbox.
+    if os.getpgid(os.getpid()) != os.getpid():
+        os.setsid()
+
     mod = module.Module(__mod__, __title__, __version__, __doc__, None,
                        (GUI_MODE,), (UI_TOOLKIT_QT4, UI_TOOLKIT_QT3))
 
@@ -85,7 +95,11 @@ if __name__ == '__main__':
         os.close(w1)
 
         if ui_toolkit == 'qt3':
-            import ui.systemtray as systray
+            try:
+                import ui.systemtray as systray
+            except ImportError:
+                log.error("Unable to load Qt3 support. Is it installed?")
+                sys.exit(1)                  
         
         else: # qt4
             try:
