@@ -21,7 +21,7 @@
 
 # Local
 from base.g import *
-from base import utils, pml, service
+from base import utils, pml
 from copier import copier
 
 # Qt
@@ -35,13 +35,11 @@ import Queue
 
 
 class ScrollCopyView(ScrollView):
-    def __init__(self, hpssd_sock, toolbox_hosted=True, num_copies=None, contrast=None, quality=None, 
-                reduction=None, fit_to_page=None, parent = None, form=None, name = None,fl = 0):
-        ScrollView.__init__(self,parent,name,fl)
+    def __init__(self, service, num_copies=None, contrast=None, quality=None,
+                reduction=None, fit_to_page=None, parent=None, form=None, name=None, fl=0):
+        ScrollView.__init__(self, service, parent, name, fl)
 
-        self.toolbox_hosted = toolbox_hosted
         self.form = form
-        self.sock = hpssd_sock
 
         self.num_copies = num_copies
         self.contrast = contrast
@@ -100,7 +98,7 @@ class ScrollCopyView(ScrollView):
             self.contrastDefaultPushButton.setEnabled(a != 0)
 
             # reduction/enlargement/fittopage
-            
+
             self.reductionSlider.setRange(self.max_reduction, self.max_enlargement)
             self.reductionSlider.setTickmarks(QSlider.Below)
             self.reductionSlider.setTickInterval(10)
@@ -108,7 +106,7 @@ class ScrollCopyView(ScrollView):
 
             self.reductionSpinBox.setMaxValue(self.max_enlargement)
             self.reductionSpinBox.setMinValue(self.max_reduction)
-            self.reductionSlider.setValue(self.reduction)
+            self.reductionSpinBox.setValue(self.reduction)
             self.reductionSpinBox.setSuffix("%")
 
             if self.fit_to_page == pml.COPIER_FIT_TO_PAGE_ENABLED:
@@ -163,7 +161,7 @@ class ScrollCopyView(ScrollView):
                 s = 'Disabled' # 1
 
             log.debug("Default Fit to page: %s (%s)" % (self.fit_to_page, s))
-            log.debug("Scan style (models.dat: scan-style): %d" % self.scan_style)            
+            log.debug("Scan style (models.dat: scan-style): %d" % self.scan_style)
 
         finally:
             self.dev.closePML()
@@ -187,14 +185,9 @@ class ScrollCopyView(ScrollView):
 
         self.addGroupHeading("space1", "")
 
-        if self.toolbox_hosted:
-            s = self.__tr("<< Functions")
-        else:
-            s = self.__tr("Close")
-
-        self.copyButton = self.addActionButton("bottom_nav", self.__tr("Make Copies(s)"), 
-                                self.copyButton_clicked, 'print.png', 'print-disabled.png', 
-                                s, self.funcButton_clicked)
+        self.copyButton = self.addActionButton("bottom_nav", self.__tr("Make Copies(s)"),
+                                self.copyButton_clicked, 'print.png', 'print.png',
+                                self.__tr("Close"), self.funcButton_clicked)
 
 
 
@@ -204,9 +197,8 @@ class ScrollCopyView(ScrollView):
     def onDeviceChange(self, cur_device=None):
         ScrollView.onDeviceChange(self, cur_device)
 
-        self.dev = copier.PMLCopyDevice(device_uri=self.cur_device.device_uri, 
-                                        printer_name=self.cur_printer, 
-                                        hpssd_sock=self.sock)
+        self.dev = copier.PMLCopyDevice(device_uri=self.cur_device.device_uri,
+                                        printer_name=self.cur_printer)
 
         self.scan_style = self.dev.mq.get('scan-style', SCAN_STYLE_FLATBED)
         self.copy_type = self.dev.mq.get('copy-type', COPY_TYPE_DEVICE)
@@ -319,6 +311,9 @@ class ScrollCopyView(ScrollView):
         layout43.addWidget(self.reductionSlider,0,2)
 
         self.reductionSpinBox = QSpinBox(widget, "reductionSpinBox")
+        self.reductionSpinBox.setMaxValue(100)
+        self.reductionSpinBox.setMinValue(0)
+        self.reductionSpinBox.setValue(100)
         self.reductionSpinBox.setSuffix("%")
         layout43.addWidget(self.reductionSpinBox,0,3)
 
@@ -414,7 +409,7 @@ class ScrollCopyView(ScrollView):
         self.connect(self.contrastSpinBox, SIGNAL("valueChanged(int)"), self.contrastSpinBox_valueChanged)
         self.connect(self.contrastDefaultPushButton, SIGNAL("clicked()"), self.contrastDefaultPushButton_clicked)
 
-        self.addWidget(widget, "contrast")        
+        self.addWidget(widget, "contrast")
 
 
     def contrastSlider_valueChanged(self, a0):
@@ -441,8 +436,7 @@ class ScrollCopyView(ScrollView):
 
     def copy_canceled(self):
         self.event_queue.put(copier.COPY_CANCELED)
-        service.sendEvent(self.sock, EVENT_COPY_JOB_CANCELED, device_uri=self.device_uri)
-
+        self.dev.sendEvent(EVENT_COPY_JOB_CANCELED)
 
     def copy_timer_timeout(self):
         while self.update_queue.qsize():
@@ -454,34 +448,41 @@ class ScrollCopyView(ScrollView):
             if status == copier.STATUS_IDLE:
                 self.copy_timer.stop()
 
-                self.pb.hide()
-                self.form.statusBar().removeWidget(self.pb)
+                #self.pb.hide()
+                #self.form.statusBar().removeWidget(self.pb)
 
             elif status in (copier.STATUS_SETTING_UP, copier.STATUS_WARMING_UP):
-                self.pb.setProgress(self.pb.progress()+1)
+                #self.pb.setProgress(self.pb.progress()+1)
+                pass
 
             elif status == copier.STATUS_ACTIVE:
-                self.pb.setProgress(self.pb.progress()+1)
+                #self.pb.setProgress(self.pb.progress()+1)
+                pass
 
             elif status in (copier.STATUS_ERROR, copier.STATUS_DONE):
                 self.copy_timer.stop()
-                self.pb.hide()
-                self.form.statusBar().removeWidget(self.pb)
+                #self.pb.hide()
+                #self.form.statusBar().removeWidget(self.pb)
+
+                # Close the dialog box.
+                #
+                if self.waitdlg is not None:
+                    self.waitdlg.hide()
+                    self.waitdlg.close()
+                    self.waitdlg = None
 
                 if status == copier.STATUS_ERROR:
                     self.form.FailureUI(self.__tr("<b>Copier error.</b><p>"))
-                    service.sendEvent(self.sock, EVENT_COPY_JOB_FAIL, device_uri=self.cur_device.device_uri)
+                    self.dev.sendEvent(EVENT_COPY_JOB_FAIL)
 
                 elif status == copier.STATUS_DONE:
-                    service.sendEvent(self.sock, EVENT_END_COPY_JOB, device_uri=self.cur_device.device_uri)
+                    pass
+                    self.dev.sendEvent(EVENT_END_COPY_JOB)
 
                 self.cur_device.close()
                 self.copyButton.setEnabled(True)
 
-                if self.toolbox_hosted:
-                    self.form.SwitchFunctionsTab("funcs")
-                else:
-                    self.form.close()
+                self.form.close()
 
 
     def copyButton_clicked(self):
@@ -493,26 +494,26 @@ class ScrollCopyView(ScrollView):
                 self.form.FailureUI(self.__tr("<b>Cannot copy: Device is busy or not available.</b><p>Please check device and try again. [1]"))
                 return
 
-            service.sendEvent(self.sock, EVENT_START_COPY_JOB, device_uri=self.cur_device.device_uri)
-
-            self.pb = QProgressBar()
-            self.pb.setTotalSteps(2)
-            self.form.statusBar().addWidget(self.pb)
-            self.pb.show()
+            self.dev.sendEvent(EVENT_START_COPY_JOB, self.cur_printer, 0, '')
+            #self.pb = QProgressBar()
+            #self.pb.setTotalSteps(2)
+            #self.form.statusBar().addWidget(self.pb)
+            #self.pb.show()
 
             log.debug("Num copies: %d" % self.num_copies)
             log.debug("Contrast: %d" % self.contrast)
             log.debug("Reduction: %d" % self.reduction)
 
-            if self.quality == 0:
+            s = 'Normal'
+            if self.quality == pml.COPIER_QUALITY_FAST:
                 s = 'Fast'
-            elif self.quality == 1:
+            elif self.quality == pml.COPIER_QUALITY_DRAFT:
                 s = 'Draft'
-            elif self.quality == 2:
+            elif self.quality == pml.COPIER_QUALITY_NORMAL:
                 s = 'Normal'
-            elif self.quality == 3:
+            elif self.quality == pml.COPIER_QUALITY_PRESENTATION:
                 s = 'Presentation'
-            elif self.quality == 4:
+            elif self.quality == pml.COPIER_QUALITY_BEST:
                 s = 'Best'
 
             log.debug("Quality: %d (%s)" % (self.quality, s))
@@ -525,13 +526,18 @@ class ScrollCopyView(ScrollView):
             log.debug("Fit to page: %s (%s)" % (self.fit_to_page, s))
             log.debug("Scan style: %d" % self.scan_style)
 
+            # Open the dialog box.
+            #
+            self.waitdlg = WaitForm(0, self.__tr("Copying..."), self.copy_canceled, self, modal=1)
+            self.waitdlg.show()
+
             self.copy_timer = QTimer(self, "CopyTimer")
             self.connect(self.copy_timer, SIGNAL('timeout()'), self.copy_timer_timeout)
             self.copy_timer.start(1000) # 1 sec UI updates
 
             self.dev.copy(self.num_copies, self.contrast, self.reduction,
                           self.quality, self.fit_to_page, self.scan_style,
-                          self.update_queue, self.event_queue)                
+                          self.update_queue, self.event_queue)
 
         finally:
             #self.cur_device.close()
@@ -539,10 +545,8 @@ class ScrollCopyView(ScrollView):
             pass
 
     def funcButton_clicked(self):
-        if self.toolbox_hosted:
-            self.form.SwitchFunctionsTab("funcs")
-        else:
-            self.form.close()
+        self.dev.close()
+        self.form.close()
 
     def __tr(self,s,c = None):
         return qApp.translate("ScrollCopy",s,c)
