@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2001-2007 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2001-2009 Hewlett-Packard Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ import operator
 
 # Local
 from base.g import *
-from base import device, utils
+from base import device, utils, models, pkit
 from prnt import cups
 from installer import core_install
 from ui_utils import load_pixmap
@@ -79,7 +79,7 @@ class PrinterNameValidator(QValidator):
 
     def validate(self, input, pos):
         input = unicode(input)
-        
+
         if not input:
             return QValidator.Acceptable, pos
 
@@ -102,38 +102,37 @@ class PhoneNumValidator(QValidator):
 
     def validate(self, input, pos):
         input = unicode(input)
-        
+
         if not input:
             return QValidator.Acceptable, pos
-        
+
         elif input[pos-1] not in u'0123456789-(+) ':
             return QValidator.Invalid, pos
-        
+
         else:
             return QValidator.Acceptable, pos
 
 
 
 class SetupForm(SetupForm_base):
-    def __init__(self, bus, param, jd_port=1, username='', parent=None, name=None, modal=0, fl=0):
+    def __init__(self, bus, param, jd_port=1, parent=None, name=None, modal=0, fl=0):
         SetupForm_base.__init__(self, parent, name, modal, fl)
 
         self.start_page = self.ConnectionPage
         self.first_page = True
-        self.username = username
 
         if bus is None:
             self.bus = 'usb'
         else:
             self.bus = bus[0]
             self.start_page = self.ProbedDevicesPage
-            
+
         if not prop.par_build:
             self.parRadioButton.setEnabled(False)
-            
+
         if not prop.net_build:
             self.netRadioButton.setEnabled(False)
-            
+
         if not prop.par_build and not prop.net_build:
             self.bus = 'usb'
             self.start_page = self.ProbedDevicesPage
@@ -146,13 +145,13 @@ class SetupForm(SetupForm_base):
             device_uri, sane_uri, fax_uri = device.makeURI(self.param, self.jd_port)
 
             if device_uri:
-                self.device_uri = device_uri            
+                self.device_uri = device_uri
                 self.start_page = self.PPDPage
-            
+
             else:
                 self.FailureUI(self.__tr("<b>Device not found.</b> <p>Please make sure your printer is properly connected and powered-on."))
 
-        self.setIcon(load_pixmap('prog', '48x48'))
+        self.setIcon(load_pixmap('hp_logo', '128x128'))
 
         self.connectionTypeButtonGroup.setButton(0)
         self.device_uri = ''
@@ -187,13 +186,15 @@ class SetupForm(SetupForm_base):
         self.setHelpEnabled(self.PPDPage, False)
         self.setHelpEnabled(self.PrinterNamePage, False)
         self.setHelpEnabled(self.FinishedPage, False)
-        
+
         self.faxNameLineEdit.setMaxLength(50)
         self.printerNameLineEdit.setMaxLength(50)
 
         QToolTip.add(self.searchFiltersPushButton2,
             self.__tr('Current: Filter: [%2]  Search: "%3"  TTL: %4  Timeout: %5s').arg(','.join(self.filter)).arg(self.search or '').arg(self.ttl).arg(self.timeout))
 
+        cups.setPasswordCallback(showPasswordUI)
+
 
     def showPage(self, page):
         orig_page = page
@@ -212,10 +213,10 @@ class SetupForm(SetupForm_base):
         if page is self.ConnectionPage: # start --> ConnectionPage
             pass
 
-        elif page is self.ProbedDevicesPage: 
+        elif page is self.ProbedDevicesPage:
             # ConnectionPage --> ProbedDevicesPage/EnterIPPage/DeviceNotFoundPage
             devices_found = self.updateProbedDevicesPage()
-            
+
 
         elif page is self.PPDPage: # ProbedDevicesPage --> PPDPage
             if self.param:
@@ -223,38 +224,35 @@ class SetupForm(SetupForm_base):
 
                 if device_uri:
                     self.device_uri = device_uri
-            
-            back_end, is_hp, bus, model, serial, dev_file, host, port = device.parseDeviceURI(self.device_uri)
+
+            back_end, is_hp, bus, model, serial, dev_file, host, port = \
+                device.parseDeviceURI(self.device_uri)
+
             self.bus = bus
             self.mq = device.queryModelByURI(self.device_uri)
-                    
-            norm_model = device.normalizeModelName(model).lower()
-                    
+
+            norm_model = models.normalizeModelName(model).lower()
+
+            core = core_install.CoreInstall()
+            core.set_plugin_version()
             plugin = self.mq.get('plugin', PLUGIN_NONE)
-            if plugin > PLUGIN_NONE:
-                
-                plugin_lib = self.mq.get("plugin-library")
-                fw_download = self.mq.get("fw-download")
-                
-                self.core = core_install.CoreInstall()
-                if not self.core.check_for_plugin(norm_model):
-                    
-                    from pluginform import PluginForm
-                    plugin_form = PluginForm(self.core, norm_model, plugin, self.device_uri, plugin_lib, fw_download)
-                    ok = plugin_form.exec_loop()
-                    if not ok:
-                        self.reject()
+            if plugin > PLUGIN_NONE and not core.check_for_plugin():
+                ok = pkit.run_plugin_command(plugin == PLUGIN_REQUIRED)
+                if not ok or not core.check_for_plugin():
+                    if plugin == PLUGIN_REQUIRED:
+                        self.FailureUI(self.__tr("<b>The printer you are trying to setup requires a binary driver plug-in and it failed to install.</b><p>Please check your internet connection and try again.</p><p>Visit <u>http://hplipopensource.com</u> for more information.</p>"))
                         return
-                    
-                else:
-                    log.debug("Plugin support already installed")
+                    else:
+                        self.WarningUI(self.__tr("Either you have chosen to skip the installation of the optional plug-in or that installation has failed.  Your printer may not function at optimal performance."))
 
             self.updatePPDPage()
 
         elif page is self.PrinterNamePage:
             self.setDefaultPrinterName()
 
-            if fax_import_ok and prop.fax_build and self.mq.get('fax-type', FAX_TYPE_NONE) != FAX_TYPE_NONE:
+            if fax_import_ok and prop.fax_build and \
+                self.mq.get('fax-type', FAX_TYPE_NONE) not in (FAX_TYPE_NONE, FAX_TYPE_NOT_SUPPORTED):
+
                 self.faxCheckBox.setEnabled(True)
                 self.faxCheckBox.setEnabled(True)
                 self.faxNameLineEdit.setEnabled(True)
@@ -405,22 +403,22 @@ class SetupForm(SetupForm_base):
 
         if devices is None:
             FILTER_MAP = {'print' : None,
-                          'none' : None, 
-                          'scan': 'scan-type', 
-                          'copy': 'copy-type', 
+                          'none' : None,
+                          'scan': 'scan-type',
+                          'copy': 'copy-type',
                           'pcard': 'pcard-type',
                           'fax': 'fax-type',
                           }
-            
+
             filter_dict = {}
             for f in self.filter:
                 if f in FILTER_MAP:
                     filter_dict[FILTER_MAP[f]] = (operator.gt, 0)
                 else:
                     filter_dict[f] = (operator.gt, 0)
-                    
+
             devices = device.probeDevices([self.bus], self.timeout, self.ttl, filter_dict, self.search)
-            
+
             self.probeHeadingTextLabel.setText(self.__tr("%1 device(s) found on the %1:").arg(len(devices)).arg(io_str))
 
         else:
@@ -441,7 +439,7 @@ class SetupForm(SetupForm_base):
                 back_end, is_hp, bus, model, serial, dev_file, host, port = device.parseDeviceURI(d)
 
                 mq = {}
-                model_ui = device.normalizeModelUIName(model)
+                model_ui = models.normalizeModelUIName(model)
 
                 if self.bus == 'usb':
                     i = DeviceListViewItem(self.probedDevicesListView, d, mq, model_ui, serial, d)
@@ -529,24 +527,25 @@ class SetupForm(SetupForm_base):
 
         if ppds is None or not ppds:
             ppds = cups.getSystemPPDs()
-            
+
         #print ppds
 
         default_model = utils.xstrip(model.replace('series', '').replace('Series', ''), '_')
-        stripped_model = cups.stripModel(default_model)
+        stripped_model = cups.stripModel2(models.normalizeModelName(model).lower())
 
-        self.ppd_dict = cups.getPPDFile(stripped_model, ppds)
-        log.debug(self.ppd_dict)
+        self.ppd = cups.getPPDFile2(stripped_model, ppds)
+        log.debug(self.ppd)
         self.ppdListView.clear()
 
-        if self.ppd_dict:
-            for ppd in self.ppd_dict:
-                PPDListViewItem(self.ppdListView, ppd, self.ppd_dict[ppd])
+        if self.ppd is not None:
+            #for ppd in self.ppd_dict:
+            PPDListViewItem(self.ppdListView, self.ppd[0], self.ppd[1])
 
-            i = self.ppdListView.firstChild()
-            self.ppdListView.setCurrentItem(i)
-            self.ppdListView.setSelected(i, True)
-            self.ppd_file = self.ppdListView.currentItem().ppd_file
+#            i = self.ppdListView.firstChild()
+#            self.ppdListView.setCurrentItem(i)
+#            self.ppdListView.setSelected(i, True)
+#            self.ppd_file = self.ppdListView.currentItem().ppd_file
+            self.ppd_file = self.ppd[0]
             log.debug(self.ppd_file)
 
         else:
@@ -554,13 +553,15 @@ class SetupForm(SetupForm_base):
 
         QApplication.restoreOverrideCursor()
 
+
     def ppdListView_currentChanged(self,a0):
         self.ppd_file = a0.ppd_file
         log.debug(self.ppd_file)
 
 
     def otherPPDPushButton_clicked(self):
-        ppd_file = unicode(QFileDialog.getOpenFileName(sys_cfg.dirs.ppd, "PPD Files (*.ppd *.ppd.gz);;All Files (*)", self, "open file dialog", "Choose a PPD file"))
+        ppd_dir = sys_conf.get('dirs', 'ppd')
+        ppd_file = unicode(QFileDialog.getOpenFileName(ppd_dir, "PPD Files (*.ppd *.ppd.gz);;All Files (*)", self, "open file dialog", "Choose a PPD file"))
 
         if ppd_file and os.path.exists(ppd_file):
             self.updatePPDPage({ppd_file: cups.getPPDDescription(ppd_file)})
@@ -579,7 +580,7 @@ class SetupForm(SetupForm_base):
         self.installed_print_devices = device.getSupportedCUPSDevices(['hp'])
         #self.installed_print_devices = device.getSupportedCUPSDevices('*')
         log.debug(self.installed_print_devices)
-        
+
         self.installed_queues = [p.name for p in cups.getPrinters()]
 
         back_end, is_hp, bus, model, serial, dev_file, host, port = device.parseDeviceURI(self.device_uri)
@@ -605,7 +606,7 @@ class SetupForm(SetupForm_base):
         self.defaultPrinterNamePushButton.setEnabled(False)
         self.printer_name = printer_name
 
-        
+
     def setEditErrors(self):
         if self.printer_name_ok:
             self.printerNameLineEdit.setPaletteBackgroundColor(self.bg)
@@ -615,11 +616,11 @@ class SetupForm(SetupForm_base):
                 self.setNextEnabled(self.PrinterNamePage, True)
 
             QToolTip.remove(self.printerNameLineEdit)
-        
+
         else:
             self.printerNameLineEdit.setPaletteBackgroundColor(QColor(0xff, 0x99, 0x99))
             self.setNextEnabled(self.PrinterNamePage, False)
-            
+
         if self.fax_name_ok:
             self.fax_name_ok = True
             self.faxNameLineEdit.setPaletteBackgroundColor(self.bg)
@@ -628,22 +629,22 @@ class SetupForm(SetupForm_base):
                 self.setNextEnabled(self.PrinterNamePage, True)
 
             QToolTip.remove(self.faxNameLineEdit)
-        
+
         else:
             self.faxNameLineEdit.setPaletteBackgroundColor(QColor(0xff, 0x99, 0x99))
             self.setNextEnabled(self.PrinterNamePage, False)
-    
-    
+
+
     def printerNameLineEdit_textChanged(self,a0):
         self.printer_name = unicode(a0)
         self.defaultPrinterNamePushButton.setEnabled(True)
-        
+
         self.printer_name_ok = True
-        
+
         if not self.printer_name:
             QToolTip.add(self.printerNameLineEdit, self.__tr('You must enter a name for the printer.'))
             self.printer_name_ok = False
-        
+
         elif self.fax_name == self.printer_name:
             s = self.__tr('The printer name and fax name must be different. Please choose different names.')
             QToolTip.add(self.faxNameLineEdit, s)
@@ -651,20 +652,20 @@ class SetupForm(SetupForm_base):
             self.fax_name_ok = False
             self.printer_name_ok = False
             self.printer_fax_names_same = True
-        
+
         elif self.printer_name in self.installed_queues:
-            QToolTip.add(self.printerNameLineEdit, 
+            QToolTip.add(self.printerNameLineEdit,
                 self.__tr('A printer already exists with this name. Please choose a different name.'))
             self.printer_name_ok = False
-            
+
         elif self.printer_fax_names_same:
             if self.fax_name != self.printer_name:
                 self.printer_fax_names_same = False
                 self.printer_name_ok = True
-                
-                self.faxNameLineEdit.emit(SIGNAL("textChanged(const QString&)"), 
+
+                self.faxNameLineEdit.emit(SIGNAL("textChanged(const QString&)"),
                             (self.faxNameLineEdit.text(),))
-        
+
         self.setEditErrors()
 
 
@@ -717,13 +718,13 @@ class SetupForm(SetupForm_base):
     def faxNameLineEdit_textChanged(self, a0):
         self.fax_name = unicode(a0)
         self.defaultFaxNamePushButton.setEnabled(True)
-        
+
         self.fax_name_ok = True
-        
+
         if not self.fax_name:
             QToolTip.add(self.faxNameLineEdit, self.__tr('You must enter a fax name.'))
             self.fax_name_ok = False
-        
+
         elif self.fax_name == self.printer_name:
             s = self.__tr('The printer name and fax name must be different. Please choose different names.')
             QToolTip.add(self.faxNameLineEdit, s)
@@ -731,21 +732,21 @@ class SetupForm(SetupForm_base):
             self.printer_name_ok = False
             self.fax_name_ok = False
             self.printer_fax_names_same = True
-        
+
         elif self.fax_name in self.installed_queues:
-            QToolTip.add(self.faxNameLineEdit, 
+            QToolTip.add(self.faxNameLineEdit,
                 self.__tr('A fax already exists with this name. Please choose a different name.'))
             self.fax_name_ok = False
-            
+
         elif self.printer_fax_names_same:
             if self.fax_name != self.printer_name:
                 self.printer_fax_names_same = False
                 self.fax_name_ok = True
-                
-                self.printerNameLineEdit.emit(SIGNAL("textChanged(const QString&)"), 
+
+                self.printerNameLineEdit.emit(SIGNAL("textChanged(const QString&)"),
                             (self.printerNameLineEdit.text(),))
-                
-        self.setEditErrors() 
+
+        self.setEditErrors()
 
 
     def faxNumberLineEdit_textChanged(self, a0):
@@ -773,7 +774,7 @@ class SetupForm(SetupForm_base):
     def readwriteFaxInformation(self, read=True):
         try:
             QApplication.setOverrideCursor(QApplication.waitCursor)
-            
+
             d = fax.getFaxDevice(self.fax_uri, disable_dbus=True)
 
             while True:
@@ -869,24 +870,25 @@ class SetupForm(SetupForm_base):
             # TODO:
             #service.sendEvent(self.hpssd_sock, EVENT_CUPS_QUEUES_CHANGED, device_uri=self.device_uri)
             pass
-            
+
         QApplication.restoreOverrideCursor()
 
     def setupFax(self):
         QApplication.setOverrideCursor(QApplication.waitCursor)
-        
+
         if self.mq.get('fax-type', FAX_TYPE_NONE) == FAX_TYPE_SOAP:
             fax_ppd_name = "HP-Fax2-hplip" # Fixed width (2528 pixels) and 300dpi rendering
             nick = "HP Fax 2"
         else:
             fax_ppd_name = "HP-Fax-hplip" # Standard
             nick = "HP Fax"
-        
+
         ppds = []
 
         log.debug("Searching for fax file %s..." % fax_ppd_name)
-        
-        for f in utils.walkFiles(sys_cfg.dirs.ppd, pattern="HP-Fax*.ppd*", abs_paths=True):
+
+        ppd_dir = sys_conf.get('dirs', 'ppd')
+        for f in utils.walkFiles(ppd_dir, pattern="HP-Fax*.ppd*", abs_paths=True):
             ppds.append(f)
 
         for f in ppds:
@@ -898,21 +900,22 @@ class SetupForm(SetupForm_base):
         else:
             QApplication.restoreOverrideCursor()
             log.error("Fax PPD file not found.")
-            
+
             if QMessageBox.warning(self, self.__tr("Unable to find HP fax PPD file."),
                 self.__tr("The PPD file (%1.ppd) needed to setup the fax queue was not found.").arg(fax_ppd_name),
                 self.__tr("Browse to file..."), # button 0
                 self.__tr("Quit") # button 1
                 ) == 0: # Browse
-                
+
                 while True:
-                    fax_ppd = unicode(QFileDialog.getOpenFileName(sys_cfg.dirs.ppd, 
-                        "HP Fax PPD Files (*.ppd *.ppd.gz);;All Files (*)", self, 
+                    ppd_dir = sys_conf.get('dirs', 'ppd')
+                    fax_ppd = unicode(QFileDialog.getOpenFileName(ppd_dir,
+                        "HP Fax PPD Files (*.ppd *.ppd.gz);;All Files (*)", self,
                         "open file dialog", "Choose the fax PPD file"))
 
                     if not fax_ppd: # user hit cancel
                         return
-                    
+
                     if os.path.exists(fax_ppd):
                         n = cups.getPPDDescription(fax_ppd)
                         if n == nick:
@@ -921,17 +924,17 @@ class SetupForm(SetupForm_base):
                             self.FailureUI(self.__tr("<b>Incorrect fax PPD file.</b><p>The fax PPD file must have a nickname of '%1', not '%1'.").arg(nick).arg(n))
                     else:
                         self.FailureUI(self.__tr("<b>File not found.</b><p>hp-setup cannot find the file %1").arg(fax_ppd))
-                    
+
             else: # Quit
                 return
 
         if not os.path.exists(fax_ppd):
-            status, status_str = cups.addPrinter(self.fax_name.encode('utf8'), 
+            status, status_str = cups.addPrinter(self.fax_name.encode('utf8'),
                 self.fax_uri, self.fax_location, '', fax_ppd,  self.fax_desc)
-        else:        
-            status, status_str = cups.addPrinter(self.fax_name.encode('utf8'), 
+        else:
+            status, status_str = cups.addPrinter(self.fax_name.encode('utf8'),
                 self.fax_uri, self.fax_location, fax_ppd, '', self.fax_desc)
-            
+
         log.debug("addPrinter() returned (%d, %s)" % (status, status_str))
         self.installed_fax_devices = device.getSupportedCUPSDevices(['hpfax'])
 
@@ -976,15 +979,6 @@ class SetupForm(SetupForm_base):
                         d.close()
 
 
-        if self.username:
-            import pwd
-            user_path = pwd.getpwnam(self.username)[5]
-            user_config_file = os.path.join(user_path, '.hplip.conf')
-
-            if os.path.exists(user_config_file):
-                cfg = Config(user_config_file)
-                cfg.last_used.device_uri = self.device_uri
-
         QWizard.accept(self)
 
 
@@ -1000,5 +994,81 @@ class SetupForm(SetupForm_base):
                               QMessageBox.NoButton,
                               QMessageBox.NoButton)
 
+    def WarningUI(self, error_text):
+        QMessageBox.warning(self,
+                             self.caption(),
+                             error_text,
+                              QMessageBox.Ok,
+                              QMessageBox.NoButton,
+                              QMessageBox.NoButton)
+
     def __tr(self, s, c=None):
         return qApp.translate("SetupForm", s, c)
+
+
+
+class PasswordDialog(QDialog):
+    def __init__(self,prompt, parent=None, name=None, modal=0, fl=0):
+        QDialog.__init__(self,parent,name,modal,fl)
+
+        if not name:
+            self.setName("PasswordDialog")
+
+        passwordDlg_baseLayout = QGridLayout(self,1,1,11,6,"passwordDlg_baseLayout")
+
+        self.promptTextLabel = QLabel(self,"promptTextLabel")
+        passwordDlg_baseLayout.addMultiCellWidget(self.promptTextLabel,0,0,0,1)
+
+        self.usernameTextLabel = QLabel(self,"usernameTextLabel")
+        passwordDlg_baseLayout.addMultiCellWidget(self.usernameTextLabel,1,1,0,1)
+
+        self.usernameLineEdit = QLineEdit(self,"usernameLineEdit")
+        self.usernameLineEdit.setEchoMode(QLineEdit.Normal)
+        passwordDlg_baseLayout.addMultiCellWidget(self.usernameLineEdit,1,1,1,2)
+
+        self.passwordTextLabel = QLabel(self,"passwordTextLabel")
+        passwordDlg_baseLayout.addMultiCellWidget(self.passwordTextLabel,2,2,0,1)
+
+        self.passwordLineEdit = QLineEdit(self,"passwordLineEdit")
+        self.passwordLineEdit.setEchoMode(QLineEdit.Password)
+        passwordDlg_baseLayout.addMultiCellWidget(self.passwordLineEdit,2,2,1,2)
+
+        self.okPushButton = QPushButton(self,"okPushButton")
+        passwordDlg_baseLayout.addWidget(self.okPushButton,3,2)
+
+        self.languageChange()
+
+        self.resize(QSize(420,163).expandedTo(self.minimumSizeHint()))
+        self.clearWState(Qt.WState_Polished)
+
+        self.connect(self.okPushButton,SIGNAL("clicked()"),self.accept)
+        self.connect(self.passwordLineEdit,SIGNAL("returnPressed()"),self.accept)
+    def getUsername(self):
+        return unicode(self.usernameLineEdit.text())
+
+    def getPassword(self):
+        return unicode(self.passwordLineEdit.text())
+
+    def languageChange(self):
+        self.setCaption(self.__tr("HP Device Manager - Enter Password"))
+        self.promptTextLabel.setText(self.__tr("You do not have authorization for this function."))
+        self.usernameTextLabel.setText(self.__tr("Username"))
+        self.passwordTextLabel.setText(self.__tr("Password"))
+        self.okPushButton.setText(self.__tr("OK"))
+
+    def __tr(self,s,c = None):
+        return qApp.translate("PasswordDialog",s,c)
+
+
+
+def showPasswordUI(prompt):
+    try:
+        dlg = PasswordDialog(prompt, None)
+
+        if dlg.exec_loop() == QDialog.Accepted:
+            return (dlg.getUsername(), dlg.getPassword())
+
+    finally:
+        pass
+
+    return ("", "")
