@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2001-2008 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2001-2009 Hewlett-Packard Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,10 +25,12 @@ import operator
 
 # Local
 from base.g import *
-from base import device, utils,  models
+from base import device, utils, models, pkit
 from prnt import cups
 from base.codes import *
 from ui_utils import *
+#from installer import core_install
+from installer.core_install import CoreInstall
 
 # Qt
 from PyQt4.QtCore import *
@@ -36,6 +38,7 @@ from PyQt4.QtGui import *
 
 # Ui
 from setupdialog_base import Ui_Dialog
+from plugindialog import PluginDialog
 
 # Fax
 try:
@@ -50,11 +53,13 @@ except ImportError:
 PAGE_DISCOVERY = 0
 PAGE_DEVICES = 1
 PAGE_ADD_PRINTER = 2
-PAGE_MAX = 2
+PAGE_REMOVE = 3
+
 
 BUTTON_NEXT = 0
 BUTTON_FINISH = 1
 BUTTON_ADD_PRINTER = 2
+BUTTON_REMOVE = 3
 
 ADVANCED_SHOW = 0
 ADVANCED_HIDE = 1
@@ -65,6 +70,79 @@ DEVICE_DESC_MULTI_FUNC = 2
 
 
 
+
+
+class PasswordDialog(QDialog):
+    def __init__(self, prompt, parent=None, name=None, modal=0, fl=0):
+        QDialog.__init__(self, parent)
+
+        Layout= QGridLayout(self)
+        Layout.setMargin(11)
+        Layout.setSpacing(6)
+
+        self.PromptTextLabel = QLabel(self)
+        Layout.addWidget(self.PromptTextLabel,0,0,1,3)
+
+        self.UsernameTextLabel = QLabel(self)
+        Layout.addWidget(self.UsernameTextLabel,1,0)
+
+        self.UsernameLineEdit = QLineEdit(self)
+        self.UsernameLineEdit.setEchoMode(QLineEdit.Normal)
+        Layout.addWidget(self.UsernameLineEdit,1,1,1,2)
+
+        self.PasswordTextLabel = QLabel(self)
+        Layout.addWidget(self.PasswordTextLabel,2,0)
+
+        self.PasswordLineEdit = QLineEdit(self)
+        self.PasswordLineEdit.setEchoMode(QLineEdit.Password)
+        Layout.addWidget(self.PasswordLineEdit,2,1,1,2)
+
+        self.OkPushButton = QPushButton(self)
+        Layout.addWidget(self.OkPushButton,3,2)
+
+        self.languageChange()
+
+        self.resize(QSize(420,163).expandedTo(self.minimumSizeHint()))
+
+        self.connect(self.OkPushButton, SIGNAL("clicked()"), self.accept)
+        self.connect(self.PasswordLineEdit, SIGNAL("returnPressed()"), self.accept)
+
+
+    def getUsername(self):
+        return unicode(self.UsernameLineEdit.text())
+
+
+    def getPassword(self):
+        return unicode(self.PasswordLineEdit.text())
+
+
+    def languageChange(self):
+        self.setWindowTitle(self.__tr("HP Device Manager - Enter Password"))
+        self.PromptTextLabel.setText(self.__tr("You do not have authorization for this function."))
+        self.UsernameTextLabel.setText(self.__tr("Username:"))
+        self.PasswordTextLabel.setText(self.__tr("Password:"))
+        self.OkPushButton.setText(self.__tr("OK"))
+
+
+    def __tr(self,s,c = None):
+        return qApp.translate("SetupDialog",s,c)
+
+
+
+def showPasswordUI(prompt):
+    try:
+        dlg = PasswordDialog(prompt, None)
+
+        if dlg.exec_() == QDialog.Accepted:
+            return (dlg.getUsername(), dlg.getPassword())
+
+    finally:
+        pass
+
+    return ("", "")
+
+
+
 class DeviceTableWidgetItem(QTableWidgetItem):
     def __init__(self, text, device_uri):
         QTableWidgetItem.__init__(self, text, QTableWidgetItem.UserType)
@@ -73,28 +151,37 @@ class DeviceTableWidgetItem(QTableWidgetItem):
 
 
 class SetupDialog(QDialog, Ui_Dialog):
-    def __init__(self, parent, param, jd_port, username):
+    def __init__(self, parent, param, jd_port, device_uri=None, remove=False):
         QDialog.__init__(self, parent)
         self.setupUi(self)
 
         self.param = param
         self.jd_port = jd_port
-        self.username = username
-        self.device_uri = None
+        self.device_uri = device_uri
+        self.remove = remove
+
+        if device_uri:
+            log.info("Using device: %s" % device_uri)
 
         self.initUi()
 
-        if self.skip_discovery:
-            QTimer.singleShot(0, self.showDevicesPage)
+        if self.remove:
+            QTimer.singleShot(0, self.showRemovePage)
         else:
-            QTimer.singleShot(0, self.showDiscoveryPage)
+            if self.skip_discovery:
+                QTimer.singleShot(0, self.showDevicesPage)
+            else:
+                QTimer.singleShot(0, self.showDiscoveryPage)
+
+        cups.setPasswordCallback(showPasswordUI)
+
 
     #
     # INIT
     #
 
     def initUi(self):
-        self.setWindowIcon(QIcon(load_pixmap('prog', '48x48')))
+        self.setWindowIcon(QIcon(load_pixmap('hp_logo', '128x128')))
 
         # connect signals/slots
         self.connect(self.CancelButton, SIGNAL("clicked()"), self.CancelButton_clicked)
@@ -102,9 +189,14 @@ class SetupDialog(QDialog, Ui_Dialog):
         self.connect(self.NextButton, SIGNAL("clicked()"), self.NextButton_clicked)
         self.connect(self.ManualGroupBox,  SIGNAL("clicked(bool)"),  self.ManualGroupBox_clicked)
 
-        self.initDiscoveryPage()
-        self.initDevicesPage()
-        self.initAddPrinterPage()
+        if self.remove:
+            self.initRemovePage()
+            self.max_page = 1
+        else:
+            self.initDiscoveryPage()
+            self.initDevicesPage()
+            self.initAddPrinterPage()
+            self.max_page = PAGE_ADD_PRINTER
 
     #
     #  DISCOVERY PAGE
@@ -162,6 +254,29 @@ class SetupDialog(QDialog, Ui_Dialog):
                         else:
                             FailureUI(self, self.__tr("<b>Invalid manual discovery parameter.</b>"))
 
+        elif self.device_uri: # If device URI specified on the command line, skip discovery
+                              # if the device URI is well-formed (but not necessarily valid)
+            try:
+                back_end, is_hp, self.bus, model, serial, dev_file, host, port = \
+                device.parseDeviceURI(self.device_uri)
+
+            except Error:
+                log.error("Invalid device URI specified: %s" % self.device_uri)
+
+            else:
+                name = ''
+                if self.bus == 'net':
+                    try:
+                        log.debug("Trying to get hostname for device...")
+                        name = socket.gethostbyaddr(host)[0]
+                    except socket.herror:
+                        log.debug("Failed.")
+                    else:
+                        log.debug("Host name=%s" % name)
+
+                self.devices = {self.device_uri : (model, model, name)}
+                self.skip_discovery = True
+
         # If no network or parallel, usb is only option, skip initial page...
         elif not prop.par_build and not prop.net_build:
             self.skip_discovery = True
@@ -169,6 +284,7 @@ class SetupDialog(QDialog, Ui_Dialog):
             self.UsbRadioButton.setChecked(True)
             self.setUsbRadioButton(True)
 
+
         self.DeviceTypeComboBox.addItem("All devices/printers", QVariant(DEVICE_DESC_ALL))
         self.DeviceTypeComboBox.addItem("Single function printers only", QVariant(DEVICE_DESC_SINGLE_FUNC))
         self.DeviceTypeComboBox.addItem("All-in-one/MFP devices only", QVariant(DEVICE_DESC_MULTI_FUNC))
@@ -328,8 +444,7 @@ class SetupDialog(QDialog, Ui_Dialog):
 
     def initDevicesPage(self):
         self.connect(self.RefreshButton,  SIGNAL("clicked()"),  self.RefreshButton_clicked)
-        #filename =  "/home/dwelch/svn/trunk/src/data/images/other/hp-tux-printer.png"
-        #self.DevicesTableWidget.setStyleSheet('background-image: url("%s"); background-attachment: fixed;' % filename)
+
 
     def showDevicesPage(self):
         self.BackButton.setEnabled(True)
@@ -424,6 +539,8 @@ class SetupDialog(QDialog, Ui_Dialog):
 
         self.DevicesTableWidget.resizeColumnsToContents()
         self.DevicesTableWidget.selectRow(0)
+        self.DevicesTableWidget.setSortingEnabled(True)
+        self.DevicesTableWidget.sortItems(0)
 
 
     def clearDevicesTable(self):
@@ -467,7 +584,23 @@ class SetupDialog(QDialog, Ui_Dialog):
 
 
     def showAddPrinterPage(self):
+        # Install the plugin if needed...
+        core = CoreInstall()
+        plugin = self.mq.get('plugin', PLUGIN_NONE)
+        if plugin > PLUGIN_NONE:
+
+            if not core.check_for_plugin():
+                ok = pkit.run_plugin_command(plugin == PLUGIN_REQUIRED)
+
+                if not ok or not core.check_for_plugin():
+                    if plugin == PLUGIN_REQUIRED:
+                        FailureUI(self, self.__tr("<b>The printer you are trying to setup requires a binary driver plug-in and it failed to install.</b><p>Please check your internet connection and try again.</p><p>Visit <u>http://hplipopensource.com</u> for more infomation.</p>"))
+                        return
+                    else:
+                        WarningUI(self, self.__tr("Either you have chosen to skip the installation of the optional plug-in or that installation has failed.  Your printer may not function at optimal performance."))
+
         self.setNextButton(BUTTON_ADD_PRINTER)
+
         if not self.printer_name:
             self.setDefaultPrinterName()
 
@@ -500,50 +633,45 @@ class SetupDialog(QDialog, Ui_Dialog):
         self.displayPage(PAGE_ADD_PRINTER)
 
 
+
+
     def updatePPD(self):
         if self.print_ppd is None:
             log.error("No appropriate print PPD file found for model %s" % self.model)
             self.PPDFileLineEdit.setText(self.__tr('(Not found. Click browse button to select a PPD file.)'))
+            try:
+                self.PPDFileLineEdit.setStyleSheet("background-color: yellow")
+            except AttributeError:
+                pass
             self.PrinterDescriptionLineEdit.setText(QString(""))
 
         else:
             self.PPDFileLineEdit.setText(self.print_ppd[0])
             self.PrinterDescriptionLineEdit.setText(self.print_ppd[1])
+            try:
+                self.PPDFileLineEdit.setStyleSheet("")
+            except AttributeError:
+                pass
 
 
     def OtherPPDButton_clicked(self, b):
         ppd_file = unicode(QFileDialog.getOpenFileName(self, self.__tr("Select PPD File"),
-                                                       sys_cfg.dirs.ppd,
+                                                       sys_conf.get('dirs', 'ppd'),
                                                        self.__tr("PPD Files (*.ppd *.ppd.gz);;All Files (*)")))
 
         if ppd_file and os.path.exists(ppd_file):
             self.print_ppd = (ppd_file, cups.getPPDDescription(ppd_file))
             self.updatePPD()
+            self.setAddPrinterButton()
 
 
     def findPrinterPPD(self):
         QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
         try:
-            log.debug("Searching for print PPD for model %s" % self.model)
+            self.print_ppd = None
             self.ppds = cups.getSystemPPDs()
-            model = self.model.replace('hp_', '').replace('hp-', '')
-            found_ppds = {}
-
-            for p in self.ppds:
-                if model in p:
-                    print "Found print PPD: %s (%s)" % (p, self.ppds[p])
-                    found_ppds[p] = self.ppds[p]
-
-            if len(found_ppds) == 0:
-                self.print_ppd = None
-
-            elif len(found_ppds) == 1:
-                self.print_ppd = found_ppds.items()[0] # (filename, description)
-
-            elif len(found_ppds) > 1:
-                ppd = cups.getPPDFile(self.model, found_ppds)
-                self.print_ppd = ppd.items()[0]
-
+            model = cups.stripModel2(self.model)
+            self.print_ppd = cups.getPPDFile2(model, self.ppds)
         finally:
             QApplication.restoreOverrideCursor()
 
@@ -562,7 +690,7 @@ class SetupDialog(QDialog, Ui_Dialog):
 
             ppds = []
 
-            for f in utils.walkFiles(sys_cfg.dirs.ppd, pattern="HP-Fax*.ppd*", abs_paths=True):
+            for f in utils.walkFiles(sys_conf.get('dirs', 'ppd'), pattern="HP-Fax*.ppd*", abs_paths=True):
                 ppds.append(f)
 
             for f in ppds:
@@ -702,15 +830,27 @@ class SetupDialog(QDialog, Ui_Dialog):
     def setIndicators(self):
         if self.printer_name_ok:
             self.PrinterNameLineEdit.setToolTip(QString(""))
-            self.PrinterNameLineEdit.setStyleSheet("")
+            try:
+                self.PrinterNameLineEdit.setStyleSheet("")
+            except AttributeError:
+                pass
         else:
-            self.PrinterNameLineEdit.setStyleSheet("background-color: yellow")
+            try:
+                self.PrinterNameLineEdit.setStyleSheet("background-color: yellow")
+            except AttributeError:
+                pass
 
         if self.fax_name_ok:
             self.FaxNameLineEdit.setToolTip(QString(""))
-            self.PrinterNameLineEdit.setStyleSheet("")
+            try:
+                self.PrinterNameLineEdit.setStyleSheet("")
+            except AttributeError:
+                pass
         else:
-            self.PrinterNameLineEdit.setStyleSheet("background-color: yellow")
+            try:
+                self.PrinterNameLineEdit.setStyleSheet("background-color: yellow")
+            except AttributeError:
+                pass
 
 
     def setAddPrinterButton(self):
@@ -811,7 +951,7 @@ class SetupDialog(QDialog, Ui_Dialog):
                     error_text = self.__tr("Unable to communicate with the device. Please check the device and try again.")
                     log.error(unicode(error_text))
                     if QMessageBox.critical(self,
-                                           self.caption(),
+                                           self.windowTitle(),
                                            error_text,
                                            QMessageBox.Retry | QMessageBox.Default,
                                            QMessageBox.Cancel | QMessageBox.Escape,
@@ -839,7 +979,7 @@ class SetupDialog(QDialog, Ui_Dialog):
                                 log.error(unicode(error_text))
 
                                 if QMessageBox.critical(self,
-                                                       self.caption(),
+                                                       self.windowTitle(),
                                                        error_text,
                                                        QMessageBox.Retry | QMessageBox.Default,
                                                        QMessageBox.Cancel | QMessageBox.Escape,
@@ -874,13 +1014,13 @@ class SetupDialog(QDialog, Ui_Dialog):
         try:
             d = device.Device(self.device_uri)
         except Error, e:
-            self.FailureUI(self.__tr("<b>Device error:</b><p>%s (%s)." % (e.msg, e.opt)))
+            FailureUI(self, self.__tr("<b>Device error:</b><p>%s (%s)." % (e.msg, e.opt)))
 
         else:
             try:
                 d.open()
             except Error:
-                self.FailureUI(self.__tr("<b>Unable to print to printer.</b><p>Please check device and try again."))
+                FailureUI(self, self.__tr("<b>Unable to print to printer.</b><p>Please check device and try again."))
             else:
                 if d.isIdleAndNoError():
                     d.close()
@@ -889,13 +1029,91 @@ class SetupDialog(QDialog, Ui_Dialog):
                         d.printTestPage(self.printer_name)
                     except Error, e:
                         if e.opt == ERROR_NO_CUPS_QUEUE_FOUND_FOR_DEVICE:
-                            self.FailureUI(self.__tr("<b>No CUPS queue found for device.</b><p>Please install the printer in CUPS and try again."))
+                            FailureUI(self, self.__tr("<b>No CUPS queue found for device.</b><p>Please install the printer in CUPS and try again."))
                         else:
-                            self.FailureUI(self.__tr("<b>Printer Error</b><p>An error occured: %s (code=%d)." % (e.msg, e.opt)))
+                            FailureUI(self, self.__tr("<b>Printer Error</b><p>An error occured: %s (code=%d)." % (e.msg, e.opt)))
                 else:
-                    self.FailureUI(self.__tr("<b>Printer Error.</b><p>Printer is busy, offline, or in an error state. Please check the device and try again."))
+                    FailureUI(self, self.__tr("<b>Printer Error.</b><p>Printer is busy, offline, or in an error state. Please check the device and try again."))
                     d.close()
 
+    #
+    # Remove Page
+    #
+
+    def initRemovePage(self):
+        pass
+
+
+    def showRemovePage(self):
+        self.displayPage(PAGE_REMOVE)
+        self.StepText.setText(self.__tr("Step 1 of 1"))
+        self.setNextButton(BUTTON_REMOVE)
+        self.BackButton.setEnabled(False)
+        self.NextButton.setEnabled(False)
+
+        self.RemoveDevicesTableWidget.verticalHeader().hide()
+
+        self.installed_printers = device.getSupportedCUPSPrinters(['hp', 'hpfax'])
+        log.debug(self.installed_printers)
+
+        if not self.installed_printers:
+            FailureUI(self, self.__tr("<b>No printers or faxes found to remove.</b><p>You must setup a least one printer or fax before you can remove it."))
+            self.close()
+            return
+
+        self.RemoveDevicesTableWidget.setRowCount(len(self.installed_printers))
+
+        headers = [self.__tr("Select"), self.__tr('Printer (Queue) Name'), self.__tr('Type'), self.__tr('Device URI')]
+
+        self.RemoveDevicesTableWidget.setColumnCount(len(headers))
+        self.RemoveDevicesTableWidget.setHorizontalHeaderLabels(headers)
+        flags = Qt.ItemIsSelectable | Qt.ItemIsEnabled
+
+        row = 0
+        for p in self.installed_printers:
+            widget = QCheckBox(self.RemoveDevicesTableWidget)
+            self.connect(widget, SIGNAL("stateChanged(int)"), self.CheckBox_stateChanged)
+            self.RemoveDevicesTableWidget.setCellWidget(row, 0, widget)
+
+            back_end, is_hp, bus, model, serial, dev_file, host, port = \
+                device.parseDeviceURI(p.device_uri)
+
+            if self.device_uri is not None and self.device_uri == p.device_uri:
+                widget.setCheckState(Qt.Checked)
+
+            i = QTableWidgetItem(QString(p.name))
+            i.setFlags(flags)
+            i.setData(Qt.UserRole, QVariant(p.name))
+            self.RemoveDevicesTableWidget.setItem(row, 1, i)
+
+            if back_end == 'hpfax':
+                typ = self.__tr("Fax")
+            else:
+                typ = self.__tr("Printer")
+
+            i = QTableWidgetItem(typ)
+            i.setFlags(flags)
+            self.RemoveDevicesTableWidget.setItem(row, 2, i)
+
+            i = QTableWidgetItem(QString(p.device_uri))
+            i.setFlags(flags)
+            self.RemoveDevicesTableWidget.setItem(row, 3, i)
+
+            row += 1
+
+        self.RemoveDevicesTableWidget.resizeColumnsToContents()
+
+
+    def CheckBox_stateChanged(self, i):
+        for row in xrange(self.RemoveDevicesTableWidget.rowCount()):
+            widget = self.RemoveDevicesTableWidget.cellWidget(row, 0)
+            if widget.checkState() == Qt.Checked:
+                self.NextButton.setEnabled(True)
+                break
+        else:
+            self.NextButton.setEnabled(False)
+
+
     #
     # Misc
     #
@@ -929,6 +1147,17 @@ class SetupDialog(QDialog, Ui_Dialog):
             self.fax_number = unicode(self.FaxNumberLineEdit.text()).encode('utf8')
             self.addPrinter()
 
+        elif p == PAGE_REMOVE:
+            for row in xrange(self.RemoveDevicesTableWidget.rowCount()):
+                widget = self.RemoveDevicesTableWidget.cellWidget(row, 0)
+                if widget.checkState() == Qt.Checked:
+                    item = self.RemoveDevicesTableWidget.item(row, 1)
+                    printer = unicode(item.data(Qt.UserRole).toString()).encode('utf-8')
+                    log.debug("Removing printer: %s" % printer)
+                    cups.delPrinter(printer)
+
+            self.close()
+
         else:
             log.error("Invalid page!") # shouldn't happen!
 
@@ -951,8 +1180,8 @@ class SetupDialog(QDialog, Ui_Dialog):
 
 
     def displayPage(self, page):
-        self.updateStepText(page)
         self.StackedWidget.setCurrentIndex(page)
+        self.updateStepText(page)
 
 
     def setNextButton(self, typ=BUTTON_FINISH):
@@ -962,10 +1191,12 @@ class SetupDialog(QDialog, Ui_Dialog):
             self.NextButton.setText(self.__tr("Next >"))
         elif typ == BUTTON_FINISH:
             self.NextButton.setText(self.__tr("Finish"))
+        elif typ == BUTTON_REMOVE:
+            self.NextButton.setText(self.__tr("Remove"))
 
 
     def updateStepText(self, p):
-        self.StepText.setText(self.__tr("Step %1 of %2").arg(p+1).arg(PAGE_MAX+1))
+        self.StepText.setText(self.__tr("Step %1 of %2").arg(p+1).arg(self.max_page+1))
 
 
     def __tr(self,s,c = None):
